import axios from "axios"
import faker from "faker"
import { Runner } from '../models/Runner'
import { RunnerGroup } from '../models/RunnerGroup'

const baseurl = "http://localhost:4010"

axios.interceptors.request.use((config) => {
    config.headers['request-startTime'] = process.hrtime()
    return config
})

axios.interceptors.response.use((response) => {
    const start = response.config.headers['request-startTime']
    const end = process.hrtime(start)
    const milliseconds = Math.round((end[0] * 1000) + (end[1] / 1000000))
    response.headers['request-duration'] = milliseconds
    return response
})

function generateRunners(amount: number): Runner[] {
    let runners: Runner[] = new Array<Runner>();
    let group = new RunnerGroup();
    let runner = new Runner();
    for (var i = 0; i < amount; i++) {
        group.name = faker.company.bsBuzz();
        group.id = Math.floor(Math.random() * (9999999 - 1) + 1);
        runner.firstname = faker.name.firstName();
        runner.lastname = faker.name.lastName();
        runner.id = Math.floor(Math.random() * (9999999 - 1) + 1);
        runners.push(runner);
    }
    return runners;
}

async function postContracts(runners: Runner[]): Promise<Measurement> {
    const res = await axios.post(`${baseurl}/contracts`, runners);
    return new Measurement("contract", runners.length, parseInt(res.headers['request-duration']))
}

async function testContracts(sizes): Promise<Measurement[]> {
    let measurements = new Array<Measurement>();
    console.log("#### Testing contracts ####");

    for (let size of sizes) {
        const m = await postContracts(generateRunners(size));
        console.log(m.toString());
        measurements.push(m);
    }
    return measurements;
}

async function main() {
    const sizes = [0, 1, 10, 50, 100, 200, 500, 1000]
    console.log("########### Speedtest ###########");
    console.log(`Document server version (according to the api): ${(await axios.get("http://localhost:4010/version")).data.version}`);
    console.log("####### Running tests #######");
    const contractResults = await testContracts(sizes);
    console.log("####### Results #######");
    console.table(contractResults);
}

main();

class Measurement {
    public type: string;
    public inputcount: number;
    public responsetime: number;

    constructor(type: string, input: number, time: number) {
        this.type = type;
        this.inputcount = input;
        this.responsetime = time;
    }

    public toString(): string {
        return `It took ${this.responsetime}ms to generate ${this.inputcount} pdfs for the type ${this.type}.`
    }
}